<?php

namespace Bixo\Opportunity\Http\Controllers;

use Exception;
use Bixo\Opportunity\Actions\OfferWorkflow;
use Bixo\Opportunity\Http\Requests\OfferWorkflowRequest;
use Bixo\Opportunity\Http\Resources\OfferResource;
use Bixo\Opportunity\Models\Offer;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class OfferWorkflowController extends BaseController
{
    /**
     * Action controller function for offer.
     *
     * @param Model $offer
     * @param action next action for the offer.
     *
     * @return Response
     */
    public function __invoke(OfferWorkflowRequest $request, Offer $offer, $trasition)
    {
        try {
            $request = $request->all();
            $offer = OfferWorkflow::run($trasition, $offer, $request);
            $data = new OfferResource($offer);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('opportunity::offer.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('opportunity/offer/' . $offer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/offer/' . $offer->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the offer and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $offer
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $offer, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $offer = Offer::findBySignedId($offer);
        $transitions = $this->transitions($offer, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('opportunity::offer.name') . ' ' . trans('Approvals'))
            ->view('opportunity::public.offer.workflow')
            ->layout('mobile')
            ->data(compact('offer', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the offer.
     *
     * @param PublicRequest $request
     * @param string $offer
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $offer, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $offer = Offer::findBySignedId($offer);
        $user = user();
        $offer = OfferWorkflow::run($transition, $offer, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($offer, $user)
    {
        $transitions = [];
        foreach ($offer->workflow()->transitions($offer) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.offer.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'offer' => $offer->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $offer->workflow()->form($value);
            $array['label'] = trans('opportunity::offer.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
