<?php

namespace Bixo\Opportunity\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Opportunity\Forms\Viewing as ViewingForm;
use Bixo\Opportunity\Http\Requests\ViewingResourceRequest;
use Bixo\Opportunity\Http\Resources\ViewingResource;
use Bixo\Opportunity\Http\Resources\ViewingsCollection;
use Bixo\Opportunity\Models\Viewing;
use Bixo\Opportunity\Actions\ViewingAction;
use Bixo\Opportunity\Actions\ViewingActions;


/**
 * Resource controller class for viewing.
 */
class ViewingResourceController extends BaseController
{

    /**
     * Initialize viewing resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ViewingForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.opportunity.modules'), 'opportunity', guard_url('opportunity'));
            return $next($request);
        });
    }

    /**
     * Display a list of viewing.
     *
     * @return Response
     */
    public function index(ViewingResourceRequest $request)
    {
        $request = $request->all();
        $page = ViewingActions::run('paginate', $request);

        $data = new ViewingsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('opportunity::viewing.names'))
            ->view('opportunity::viewing.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display viewing.
     *
     * @param Request $request
     * @param Model   $viewing
     *
     * @return Response
     */
    public function show(ViewingResourceRequest $request, Viewing $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ViewingResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('opportunity::viewing.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('opportunity::viewing.show')
            ->output();
    }

    /**
     * Show the form for creating a new viewing.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ViewingResourceRequest $request, Viewing $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ViewingResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('opportunity::viewing.name'))
            ->view('opportunity::viewing.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new viewing.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ViewingResourceRequest $request, Viewing $model)
    {
        try {
            $request = $request->all();
            $model = ViewingAction::run('store', $model, $request);
            $data = new ViewingResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('opportunity::viewing.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('opportunity/viewing/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/opportunity/viewing'))
                ->redirect();
        }

    }

    /**
     * Show viewing for editing.
     *
     * @param Request $request
     * @param Model   $viewing
     *
     * @return Response
     */
    public function edit(ViewingResourceRequest $request, Viewing $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ViewingResource($model);
        // return view('opportunity::viewing.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('opportunity::viewing.name'))
            ->view('opportunity::viewing.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the viewing.
     *
     * @param Request $request
     * @param Model   $viewing
     *
     * @return Response
     */
    public function update(ViewingResourceRequest $request, Viewing $model)
    {
        try {
            $request = $request->all();
            $model = ViewingAction::run('update', $model, $request);
            $data = new ViewingResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('opportunity::viewing.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('opportunity/viewing/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/viewing/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the viewing.
     *
     * @param Model   $viewing
     *
     * @return Response
     */
    public function destroy(ViewingResourceRequest $request, Viewing $model)
    {
        try {

            $request = $request->all();
            $model = ViewingAction::run('destroy', $model, $request);
            $data = new ViewingResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::viewing.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('opportunity/viewing/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/viewing/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
